@@ -2,6 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
+from trytond.pyson import PYSONEncoder
 import datetime
 from trytond.tools import safe_eval
 
@@ -9,11 +10,11 @@ from trytond.tools import safe_eval
 class Template(ModelSQL, ModelView):
     _name = "product.template"
 
-    quantity = fields.Function('get_quantity', type='float', string='Quantity')
-    forecast_quantity = fields.Function('get_quantity', type='float',
-            string='Forecast Quantity')
+    quantity = fields.Function(fields.Float('Quantity'), 'get_quantity')
+    forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
+            'get_quantity')
 
-    def get_quantity(self, cursor, user, ids, name, args, context=None):
+    def get_quantity(self, cursor, user, ids, name, context=None):
         res = {}
         if name not in ('quantity', 'forecast_quantity'):
             raise Exception('Bad argument')
@@ -61,12 +62,12 @@ Template()
 class Product(ModelSQL, ModelView):
     _name = "product.product"
 
-    quantity = fields.Function('get_quantity', type='float', string='Quantity',
-            fnct_search='search_quantity')
-    forecast_quantity = fields.Function('get_quantity', type='float',
-            string='Forecast Quantity', fnct_search='search_quantity')
+    quantity = fields.Function(fields.Float('Quantity'), 'get_quantity',
+            searcher='search_quantity')
+    forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
+            'get_quantity', searcher='search_quantity')
 
-    def get_quantity(self, cursor, user, ids, name, args, context=None):
+    def get_quantity(self, cursor, user, ids, name, context=None):
         date_obj = self.pool.get('ir.date')
 
         if not (context and context.get('locations')):
@@ -97,17 +98,15 @@ class Product(ModelSQL, ModelView):
         return res
 
     def _search_quantity_eval_domain(self, line, domain):
-        res = True
-        for field, operator, operand in domain:
-            value = line.get(field)
-            if value == None:
-                return False
-            if operator not in ("=", ">=", "<=", ">", "<", "!="):
-                return False
-            if operator == "=":
-                operator= "=="
-            res = res and (safe_eval(str(value) + operator + str(operand)))
-        return res
+        field, operator, operand = domain
+        value = line.get(field)
+        if value == None:
+            return False
+        if operator not in ("=", ">=", "<=", ">", "<", "!="):
+            return False
+        if operator == "=":
+            operator= "=="
+        return (safe_eval(str(value) + operator + str(operand)))
 
     def search_quantity(self, cursor, user, name, domain=None, context=None):
         date_obj = self.pool.get('ir.date')
@@ -363,7 +362,7 @@ class Product(ModelSQL, ModelView):
         if with_childs:
             query, args = location_obj.search(cursor, user, [
                 ('parent', 'child_of', location_ids),
-                ], context=context, query_string=True)
+                ], context=context, query_string=True, order=[])
             where_clause = " IN (" + query + ") "
             where_vals = args
         else:
@@ -405,6 +404,10 @@ class Product(ModelSQL, ModelView):
             dest_clause_from = dest_clause_to =""
             dest_vals = []
 
+        # The main select clause is a union between two similar
+        # subqueries. One that sums incoming moves towards locations
+        # and on that sums outgoing moves. UNION ALL is used because
+        # we already know that there will be no duplicates.
         select_clause = \
                 "SELECT location, product, uom, sum(quantity) AS quantity "\
                 "FROM ( "\
@@ -414,7 +417,7 @@ class Product(ModelSQL, ModelView):
                     "WHERE (%s) " \
                         "AND to_location %s "\
                     "GROUP BY to_location, product ,uom "\
-                    "UNION  "\
+                    "UNION ALL "\
                     "SELECT from_location AS location, product, uom, "\
                         "-sum(quantity) AS quantity "\
                     "FROM stock_move " + product_template_join + \
@@ -574,15 +577,9 @@ class OpenLocation(Wizard):
     def _action_open_location(self, cursor, user, data, context=None):
         model_data_obj = self.pool.get('ir.model.data')
         act_window_obj = self.pool.get('ir.action.act_window')
-
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_location_quantity_tree'),
-            ('module', '=', 'stock'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
+        act_window_id = model_data_obj.get_id(cursor, user, 'stock',
+                'act_location_quantity_tree', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
 
         ctx = {}
         ctx['product'] = data['id']
@@ -590,7 +587,7 @@ class OpenLocation(Wizard):
             ctx['stock_date_end'] = data['form']['forecast_date']
         else:
             ctx['stock_date_end'] = datetime.date.max
-        res['context'] = str(ctx)
+        res['pyson_context'] = PYSONEncoder().encode(ctx)
 
         return res
 
